@@ -14,8 +14,12 @@
 
 # This class contains the basic functionality needed to run any interpreter
 # or an interpreter-based tool.
+from __future__ import annotations
+
+import os
+import sys
+import typing as T
 
-from .visitor import AstVisitor
 from .. import mparser, mesonlib
 from .. import environment
 
@@ -25,13 +29,11 @@ from ..interpreterbase import (
     InvalidArguments,
     BreakRequest,
     ContinueRequest,
+    Disabler,
     default_resolve_key,
-    TYPE_nvar,
-    TYPE_nkwargs,
 )
 
 from ..interpreter import (
-    Interpreter,
     StringHolder,
     BooleanHolder,
     IntegerHolder,
@@ -40,29 +42,34 @@ from ..interpreter import (
 )
 
 from ..mparser import (
-    AndNode,
     ArgumentNode,
     ArithmeticNode,
     ArrayNode,
     AssignmentNode,
     BaseNode,
-    ComparisonNode,
     ElementaryNode,
     EmptyNode,
-    ForeachClauseNode,
     IdNode,
-    IfClauseNode,
-    IndexNode,
     MethodNode,
     NotNode,
-    OrNode,
     PlusAssignmentNode,
     TernaryNode,
-    UMinusNode,
+    TestCaseClauseNode,
 )
 
-import os, sys
-import typing as T
+if T.TYPE_CHECKING:
+    from .visitor import AstVisitor
+    from ..interpreter import Interpreter
+    from ..interpreterbase import TYPE_nkwargs, TYPE_nvar
+    from ..mparser import (
+        AndNode,
+        ComparisonNode,
+        ForeachClauseNode,
+        IfClauseNode,
+        IndexNode,
+        OrNode,
+        UMinusNode,
+    )
 
 class DontCareObject(MesonInterpreterObject):
     pass
@@ -92,10 +99,10 @@ class AstInterpreter(InterpreterBase):
     def __init__(self, source_root: str, subdir: str, subproject: str, visitors: T.Optional[T.List[AstVisitor]] = None):
         super().__init__(source_root, subdir, subproject)
         self.visitors = visitors if visitors is not None else []
-        self.processed_buildfiles = set() # type: T.Set[str]
-        self.assignments = {}             # type: T.Dict[str, BaseNode]
-        self.assign_vals = {}             # type: T.Dict[str, T.Any]
-        self.reverse_assignment = {}      # type: T.Dict[str, BaseNode]
+        self.processed_buildfiles: T.Set[str] = set()
+        self.assignments: T.Dict[str, BaseNode] = {}
+        self.assign_vals: T.Dict[str, T.Any] = {}
+        self.reverse_assignment: T.Dict[str, BaseNode] = {}
         self.funcs.update({'project': self.func_do_nothing,
                            'test': self.func_do_nothing,
                            'benchmark': self.func_do_nothing,
@@ -103,6 +110,7 @@ class AstInterpreter(InterpreterBase):
                            'install_man': self.func_do_nothing,
                            'install_data': self.func_do_nothing,
                            'install_subdir': self.func_do_nothing,
+                           'install_symlink': self.func_do_nothing,
                            'install_emptydir': self.func_do_nothing,
                            'configuration_data': self.func_do_nothing,
                            'configure_file': self.func_do_nothing,
@@ -111,6 +119,7 @@ class AstInterpreter(InterpreterBase):
                            'add_global_arguments': self.func_do_nothing,
                            'add_global_link_arguments': self.func_do_nothing,
                            'add_project_arguments': self.func_do_nothing,
+                           'add_project_dependencies': self.func_do_nothing,
                            'add_project_link_arguments': self.func_do_nothing,
                            'message': self.func_do_nothing,
                            'generator': self.func_do_nothing,
@@ -141,18 +150,18 @@ class AstInterpreter(InterpreterBase):
                            'is_disabler': self.func_do_nothing,
                            'is_variable': self.func_do_nothing,
                            'disabler': self.func_do_nothing,
-                           'gettext': self.func_do_nothing,
                            'jar': self.func_do_nothing,
                            'warning': self.func_do_nothing,
                            'shared_module': self.func_do_nothing,
                            'option': self.func_do_nothing,
                            'both_libraries': self.func_do_nothing,
                            'add_test_setup': self.func_do_nothing,
-                           'find_library': self.func_do_nothing,
                            'subdir_done': self.func_do_nothing,
                            'alias_target': self.func_do_nothing,
                            'summary': self.func_do_nothing,
                            'range': self.func_do_nothing,
+                           'structured_sources': self.func_do_nothing,
+                           'debug': self.func_do_nothing,
                            })
 
     def _unholder_args(self, args: _T, kwargs: _V) -> T.Tuple[_T, _V]:
@@ -232,7 +241,7 @@ class AstInterpreter(InterpreterBase):
 
     def evaluate_dictstatement(self, node: mparser.DictNode) -> TYPE_nkwargs:
         def resolve_key(node: mparser.BaseNode) -> str:
-            if isinstance(node, mparser.StringNode):
+            if isinstance(node, mparser.BaseStringNode):
                 return node.value
             return '__AST_UNKNOWN__'
         arguments, kwargs = self.reduce_arguments(node.args, key_resolver=resolve_key)
@@ -247,10 +256,10 @@ class AstInterpreter(InterpreterBase):
     def evaluate_plusassign(self, node: PlusAssignmentNode) -> None:
         assert isinstance(node, PlusAssignmentNode)
         # Cheat by doing a reassignment
-        self.assignments[node.var_name] = node.value  # Save a reference to the value node
+        self.assignments[node.var_name.value] = node.value  # Save a reference to the value node
         if node.value.ast_id:
             self.reverse_assignment[node.value.ast_id] = node
-        self.assign_vals[node.var_name] = self.evaluate_statement(node.value)
+        self.assign_vals[node.var_name.value] = self.evaluate_statement(node.value)
 
     def evaluate_indexing(self, node: IndexNode) -> int:
         return 0
@@ -265,7 +274,7 @@ class AstInterpreter(InterpreterBase):
                 duplicate_key_error: T.Optional[str] = None,
             ) -> T.Tuple[T.List[TYPE_nvar], TYPE_nkwargs]:
         if isinstance(args, ArgumentNode):
-            kwargs = {}  # type: T.Dict[str, TYPE_nvar]
+            kwargs: T.Dict[str, TYPE_nvar] = {}
             for key, val in args.kwargs.items():
                 kwargs[key_resolver(key)] = val
             if args.incorrect_order():
@@ -305,17 +314,17 @@ class AstInterpreter(InterpreterBase):
         for i in node.ifs:
             self.evaluate_codeblock(i.block)
         if not isinstance(node.elseblock, EmptyNode):
-            self.evaluate_codeblock(node.elseblock)
+            self.evaluate_codeblock(node.elseblock.block)
 
     def get_variable(self, varname: str) -> int:
         return 0
 
     def assignment(self, node: AssignmentNode) -> None:
         assert isinstance(node, AssignmentNode)
-        self.assignments[node.var_name] = node.value # Save a reference to the value node
+        self.assignments[node.var_name.value] = node.value # Save a reference to the value node
         if node.value.ast_id:
             self.reverse_assignment[node.value.ast_id] = node
-        self.assign_vals[node.var_name] = self.evaluate_statement(node.value) # Evaluate the value just in case
+        self.assign_vals[node.var_name.value] = self.evaluate_statement(node.value) # Evaluate the value just in case
 
     def resolve_node(self, node: BaseNode, include_unknown_args: bool = False, id_loop_detect: T.Optional[T.List[str]] = None) -> T.Optional[T.Any]:
         def quick_resolve(n: BaseNode, loop_detect: T.Optional[T.List[str]] = None) -> T.Any:
@@ -343,7 +352,7 @@ class AstInterpreter(InterpreterBase):
             return None # Loop detected
         id_loop_detect += [node.ast_id]
 
-        # Try to evealuate the value of the node
+        # Try to evaluate the value of the node
         if isinstance(node, IdNode):
             result = quick_resolve(node)
 
@@ -356,16 +365,16 @@ class AstInterpreter(InterpreterBase):
                 result = not result
 
         elif isinstance(node, ArrayNode):
-            result = [x for x in node.args.arguments]
+            result = node.args.arguments.copy()
 
         elif isinstance(node, ArgumentNode):
-            result = [x for x in node.arguments]
+            result = node.arguments.copy()
 
         elif isinstance(node, ArithmeticNode):
             if node.operation != 'add':
                 return None # Only handle string and array concats
-            l = quick_resolve(node.left)
-            r = quick_resolve(node.right)
+            l = self.resolve_node(node.left, include_unknown_args, id_loop_detect)
+            r = self.resolve_node(node.right, include_unknown_args, id_loop_detect)
             if isinstance(l, str) and isinstance(r, str):
                 result = l + r # String concatenation detected
             else:
@@ -374,18 +383,19 @@ class AstInterpreter(InterpreterBase):
         elif isinstance(node, MethodNode):
             src = quick_resolve(node.source_object)
             margs = self.flatten_args(node.args.arguments, include_unknown_args, id_loop_detect)
-            mkwargs = {} # type: T.Dict[str, TYPE_nvar]
+            mkwargs: T.Dict[str, TYPE_nvar] = {}
+            method_name = node.name.value
             try:
                 if isinstance(src, str):
-                    result = StringHolder(src, T.cast(Interpreter, self)).method_call(node.name, margs, mkwargs)
+                    result = StringHolder(src, T.cast('Interpreter', self)).method_call(method_name, margs, mkwargs)
                 elif isinstance(src, bool):
-                    result = BooleanHolder(src, T.cast(Interpreter, self)).method_call(node.name, margs, mkwargs)
+                    result = BooleanHolder(src, T.cast('Interpreter', self)).method_call(method_name, margs, mkwargs)
                 elif isinstance(src, int):
-                    result = IntegerHolder(src, T.cast(Interpreter, self)).method_call(node.name, margs, mkwargs)
+                    result = IntegerHolder(src, T.cast('Interpreter', self)).method_call(method_name, margs, mkwargs)
                 elif isinstance(src, list):
-                    result = ArrayHolder(src, T.cast(Interpreter, self)).method_call(node.name, margs, mkwargs)
+                    result = ArrayHolder(src, T.cast('Interpreter', self)).method_call(method_name, margs, mkwargs)
                 elif isinstance(src, dict):
-                    result = DictHolder(src, T.cast(Interpreter, self)).method_call(node.name, margs, mkwargs)
+                    result = DictHolder(src, T.cast('Interpreter', self)).method_call(method_name, margs, mkwargs)
             except mesonlib.MesonException:
                 return None
 
@@ -393,7 +403,7 @@ class AstInterpreter(InterpreterBase):
         if isinstance(result, BaseNode):
             result = self.resolve_node(result, include_unknown_args, id_loop_detect)
         elif isinstance(result, list):
-            new_res = []  # type: T.List[TYPE_nvar]
+            new_res: T.List[TYPE_nvar] = []
             for i in result:
                 if isinstance(i, BaseNode):
                     resolved = self.resolve_node(i, include_unknown_args, id_loop_detect)
@@ -412,7 +422,7 @@ class AstInterpreter(InterpreterBase):
         else:
             args = [args_raw]
 
-        flattend_args = []  # type: T.List[TYPE_nvar]
+        flattened_args: T.List[TYPE_nvar] = []
 
         # Resolve the contents of args
         for i in args:
@@ -421,18 +431,21 @@ class AstInterpreter(InterpreterBase):
                 if resolved is not None:
                     if not isinstance(resolved, list):
                         resolved = [resolved]
-                    flattend_args += resolved
+                    flattened_args += resolved
             elif isinstance(i, (str, bool, int, float)) or include_unknown_args:
-                flattend_args += [i]
-        return flattend_args
+                flattened_args += [i]
+        return flattened_args
 
     def flatten_kwargs(self, kwargs: T.Dict[str, TYPE_nvar], include_unknown_args: bool = False) -> T.Dict[str, TYPE_nvar]:
-        flattend_kwargs = {}
+        flattened_kwargs = {}
         for key, val in kwargs.items():
             if isinstance(val, BaseNode):
                 resolved = self.resolve_node(val, include_unknown_args)
                 if resolved is not None:
-                    flattend_kwargs[key] = resolved
+                    flattened_kwargs[key] = resolved
             elif isinstance(val, (str, bool, int, float)) or include_unknown_args:
-                flattend_kwargs[key] = val
-        return flattend_kwargs
+                flattened_kwargs[key] = val
+        return flattened_kwargs
+
+    def evaluate_testcase(self, node: TestCaseClauseNode) -> Disabler | None:
+        return Disabler(subproject=self.subproject)
